import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:integration_test/integration_test.dart';
import 'package:kameleoon_client_flutter/src/kameleoon_client.dart';
import 'package:kameleoon_client_flutter/src/kameleoon_client_factory.dart';

import 'constants.dart';

String makeReason(String text) => "\n\n\n" + ERROR_REASON + "\n" + text + "\n \n \n";

const int featureFlagIdProduction = 149479;
const int featureFlagIdStaging = 152160;
const int featureFlagIdDevelopment = 152626;
const String featureFlagKey = "feature_flag_multi_environment_test";

void main() {
  IntegrationTestWidgetsFlutterBinding.ensureInitialized();

  testAPI(KameleoonClient client, [Function? actionWhenReady]) async {
    final completer = new Completer();

    client.runWhenReady(() async {
      await actionWhenReady?.call();
      completer.complete(COMPLETE_VALID);
    }, () {
      completer.completeError(TIMEOUT_FAIL_ERROR);
    }, TIMEOUT_MILLISECONDS);

    await expectLater(completer.future, completion(equals(COMPLETE_VALID)), reason: makeReason("runWhenReady Timeout"));
  }

  testEnvironment(
      String environment,
      int featureOkId,
      int featureCrashId1,
      int featureCrashId2,
      String featureKey,
      bool isActivateFlag) async {

    KameleoonClient client = KameleoonClientFactory.create(siteCode, environment)!;
    await testAPI(client, () async {

      try {
        bool? result = await client.activateFeature(visitorCode, featureOkId);
        if (result != isActivateFlag) {
          fail(makeReason("activateFeature(visitorCode, featureOkId)"));
        }
        result = await client.activateFeature(visitorCode, featureKey);
        if (result != isActivateFlag) {
          fail(makeReason("activateFeature(visitorCode, featureKey)"));
        }
      } catch (e) {
        fail(e.toString() +  ", No exception expected");
      }

      try {
        await client.activateFeature(visitorCode, featureCrashId1);
        fail("Exception expected");
      } catch(e) {}
      try {
        await client.activateFeature(visitorCode, featureCrashId2);
        fail("Exception expected");
      } catch(e) {}

      String expectedFeatureValue = environment;
      String key = "Key";
      try {
        String value = (await client.obtainFeatureVariable(featureOkId, key)).toString();
        if (expectedFeatureValue != value.toLowerCase()) {
          fail(makeReason("obtainFeatureVariable(featureOkId, key)"));
        }
        value = (await client.obtainFeatureVariable(featureKey, key)).toString();
        if (expectedFeatureValue != value.toLowerCase()) {
          fail(makeReason("obtainFeatureVariable(featureKey, key)"));
        }
      } catch (e) {
        fail(e.toString() +  ", No exception expected");
      }

      try {
        await client.obtainFeatureVariable(featureCrashId1, key);
        fail("Exception expected");
      } catch(e) {}
      try {
        await client.obtainFeatureVariable(featureCrashId2, key);
        fail("Exception expected");
      } catch(e) {}
    });
  }

  testWidgets("testProduction", (WidgetTester tester) async {
    if(!kIsWeb) {
      await testEnvironment(
          "production",
          featureFlagIdProduction,
          featureFlagIdStaging,
          featureFlagIdDevelopment,
          featureFlagKey,
          true);
    }
  });

  testWidgets("testStaging", (WidgetTester tester) async {
    if(kIsWeb) {
      await testEnvironment(
          "staging",
          featureFlagIdStaging,
          featureFlagIdProduction,
          featureFlagIdDevelopment,
          featureFlagKey,
          true);
    }
  });
}